import ellipsis from './ellipsis'

function makeEllipsis(el, text, position, rows, modifiers) {
  let fill = el.dataset.ellipsis
  fill = fill || '...'
  let [hasEllipsis, content] = ellipsis.make(el, text, position, fill, rows, modifiers.scale)
  el.innerHTML = content
  if (modifiers.always) {
    el.title = text
    return
  }

  if (!hasEllipsis || modifiers.none) {
    return
  }
  el.title = text
}

function render(el, {modifiers, value}, vnode) {
  let position = 'middle'
  if (modifiers.start) {
    position = 'start'
  } else if (modifiers.middle) {
    position = 'middle'
  } else if (modifiers.end) {
    position = 'end'
  }
  if (vnode.children.length !== 1) {
    throw new Error(`Ellipsis accept one text node only, got "${vnode.children.length}" children`)
  }
  if (vnode.children[0].elm) {
    if (vnode.children[0].elm.nodeType !== 3) {
      throw new Error(`Ellipsis accept text node only, got "${vnode.children[0].elm.nodeName}"`)
    }
  }
  let text = vnode.children[0].text
  if (!text) {
    return
  }
  // 值为0表示显示所有行
  if (value === 0) {
    el.innerHTML = text
    return
  }

  // 显示的行数，默认为1行
  let rows = value || 1
  text = ellipsis.clear(text)

  if (rows > 1 && position === 'middle') {
    throw new Error(`Ellipsis accept single row while position is "middle", got value "${rows}"`)
  }

  if (rows > 1 && modifiers.scale) {
    throw new Error(`Ellipsis accept single row while "scale" enabled, got value "${rows}"`)
  }
  if (!modifiers.delay) {
    makeEllipsis(el, text, position, rows, modifiers)
    return
  }
  setTimeout(() => {
    makeEllipsis(el, text, position, rows, modifiers)
  }, parseInt(el.dataset.delay) || 200)
}

function destroy(el) {
  ellipsis.destroy(el)
}

export default {
  inserted: render,
  update: render,
  unbind: destroy
}
